@@ -21,6 +21,14 @@
 
 #include <glib-object.h>
 
+#ifdef G_OS_WIN32
+#define STRICT
+#include <windows.h>
+#undef STRICT
+#undef FOCUS_EVENT		/* <windows.h> pollutes the namespace
+				 * like a six hundred pound gorilla */
+#endif
+
 #include "atk.h"
 #include "atkmarshal.h"
 #include "atk-enum-types.h"
@@ -45,6 +53,7 @@ enum
   PROP_TABLE_ROW_DESCRIPTION,
   PROP_TABLE_ROW_HEADER,
   PROP_TABLE_SUMMARY,
+  PROP_TABLE_CAPTION_OBJECT,
   PROP_LAST         /* gobject convention */
 };
 
@@ -64,7 +73,8 @@ static void            atk_object_init              (AtkObject       *accessible
                                                      AtkObjectClass  *klass);
 static AtkRelationSet* atk_object_real_ref_relation_set 
                                                     (AtkObject       *accessible);
-
+static void            atk_object_real_initialize   (AtkObject       *accessible,
+                                                     gpointer        data);
 static void            atk_object_real_set_property (GObject         *object,
                                                      guint            prop_id,
                                                      const GValue    *value,
@@ -121,6 +131,7 @@ static const gchar* atk_object_name_property_table_column_header = "accessible-t
 static const gchar* atk_object_name_property_table_row_description = "accessible-table-row-description";
 static const gchar* atk_object_name_property_table_row_header = "accessible-table-row-header";
 static const gchar* atk_object_name_property_table_summary = "accessible-table-summary";
+static const gchar* atk_object_name_property_table_caption_object = "accessible-table-caption-object";
 
 GType
 atk_object_get_type (void)
@@ -168,6 +179,7 @@ atk_object_class_init (AtkObjectClass *klass)
   klass->get_role = atk_object_real_get_role;
   klass->get_layer = atk_object_real_get_layer;
   klass->get_mdi_zorder = NULL;
+  klass->initialize = atk_object_real_initialize;
   klass->ref_state_set = atk_object_real_ref_state_set;
   klass->set_name = atk_object_real_set_name;
   klass->set_description = atk_object_real_set_description;
@@ -250,7 +262,7 @@ atk_object_class_init (AtkObjectClass *klass)
                                    PROP_TABLE_CAPTION,
                                    g_param_spec_string (atk_object_name_property_table_caption,
                                                         "Accessible Table Caption",
-                                                        "Is used to notify that the table caption has changed ",
+                                                        "Is used to notify that the table caption has changed; this property should not be used. accessible-table-caption-object should be used instead",
                                                         NULL,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
@@ -288,6 +300,13 @@ atk_object_class_init (AtkObjectClass *klass)
                                                         "Is used to notify that the table summary has changed ",
                                                         ATK_TYPE_OBJECT,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_TABLE_CAPTION_OBJECT,
+                                   g_param_spec_object (atk_object_name_property_table_caption_object,
+                                                        "Accessible Table Caption Object",
+                                                        "Is used to notify that the table caption has changed ",
+                                                        ATK_TYPE_OBJECT,
+                                                        G_PARAM_READWRITE));
   /*
    * The signal "children_changed" supports two details:
    * "add" and "remove"
@@ -743,14 +762,21 @@ atk_object_set_role (AtkObject *accessible,
                      AtkRole   role)
 {
   AtkObjectClass *klass;
+  AtkRole old_role;
 
   g_return_if_fail (ATK_IS_OBJECT (accessible));
 
   klass = ATK_OBJECT_GET_CLASS (accessible);
   if (klass->set_role)
     {
-      (klass->set_role) (accessible, role);
-      g_object_notify (G_OBJECT (accessible), atk_object_name_property_role);
+      old_role = atk_object_get_role (accessible);
+      if (old_role != role)
+        {
+          (klass->set_role) (accessible, role);
+          if (old_role != ATK_ROLE_UNKNOWN)
+          /* Do not notify for initial role setting */
+            g_object_notify (G_OBJECT (accessible), atk_object_name_property_role);
+        }
     }
 }
 
@@ -886,6 +912,14 @@ atk_object_real_set_property (GObject      *object,
       if (ATK_IS_VALUE (accessible))
         atk_value_set_current_value (ATK_VALUE (accessible), value);
       break;
+    case PROP_TABLE_SUMMARY:
+      if (ATK_IS_TABLE (accessible))
+        atk_table_set_summary (ATK_TABLE (accessible), g_value_get_object (value));
+      break;
+    case PROP_TABLE_CAPTION_OBJECT:
+      if (ATK_IS_TABLE (accessible))
+        atk_table_set_caption (ATK_TABLE (accessible), g_value_get_object (value));
+      break;
     default:
       break;
     }
@@ -927,6 +961,14 @@ atk_object_real_get_property (GObject      *object,
       if (ATK_IS_VALUE (accessible))
         atk_value_get_current_value (ATK_VALUE (accessible), value);
       break;
+    case PROP_TABLE_SUMMARY:
+      if (ATK_IS_TABLE (accessible))
+        g_value_set_object (value, atk_table_get_summary (ATK_TABLE (accessible)));
+      break;
+    case PROP_TABLE_CAPTION_OBJECT:
+      if (ATK_IS_TABLE (accessible))
+        g_value_set_object (value, atk_table_get_caption (ATK_TABLE (accessible)));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1153,6 +1195,23 @@ atk_role_get_name (AtkRole role)
   return name;
 }
 
+#ifdef G_OS_WIN32
+
+#undef ATKLOCALEDIR
+
+#define ATKLOCALEDIR get_atk_locale_dir()
+
+G_WIN32_DLLMAIN_FOR_DLL_NAME(static, dll_name)
+
+static char *
+get_atk_locale_dir (void)
+{
+  return g_win32_get_package_installation_subdirectory
+    (GETTEXT_PACKAGE, dll_name, "lib/locale");
+}
+
+#endif
+
 /**
  * atk_role_get_localized_name:
  * @role: The #AtkRole whose localized name is required
@@ -1533,3 +1592,10 @@ atk_object_remove_relationship (AtkObject       *object,
 
   return ret;
 }
+
+static void
+atk_object_real_initialize (AtkObject *accessible,
+                            gpointer  data)
+{
+  return;
+}
